package cz.cvut.kbss.ear.servlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class TestHelloWorldServlet {

    @Test public void testGetWithoutAccept() {
        final HttpServletRequest request = mock(HttpServletRequest.class);
        final HttpServletResponse response = mock(HttpServletResponse.class);
        try {

            final StringWriter w = new StringWriter();
            when(response.getWriter()).thenReturn(new PrintWriter(w));
            new HelloWorldServlet().doGet(request, response);
            w.flush();

            Assertions.assertTrue(w.toString().startsWith("HELLO"));
        } catch (IOException e) {
            Assertions.fail(e);
        }
    }

    @Test public void testGetWithJsonAccept() throws Exception {
        final HttpServletRequest request = mock(HttpServletRequest.class);
        final HttpServletResponse response = mock(HttpServletResponse.class);
        try {

            final StringWriter w = new StringWriter();
            when(response.getWriter()).thenReturn(new PrintWriter(w));
            when(request.getHeader("Accept")).thenReturn("text/json");
            new HelloWorldServlet().doGet(request, response);
            w.flush();

            final JSONParser p = new JSONParser();
            System.out.println(w.toString());
            final Object o = p.parse(w.toString());

            Assertions.assertTrue(o instanceof JSONObject);
            final JSONObject oo = (JSONObject) o;
            Assertions.assertTrue(oo.get("message").toString().startsWith("hello"));
        } catch (IOException | ParseException e) {
            Assertions.fail(e);
        }
    }
}